@@ -1,4 +1,16 @@
-#include "../fsc/fsc.h"
+/*
+Copyright 2015 Filosoft OÜ
+
+This file is part of Estnltk. It is available under the license of GPLv2 found
+in the top-level directory of this distribution and
+at http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html .
+No part of this file, may be copied, modified, propagated, or distributed
+except according to the terms contained in the license.
+
+This software is distributed on an "AS IS" basis, without warranties or conditions
+of any kind, either express or implied.
+*/
+#include "fsc.h"
 #include "ptword.h"
 
 void CPTWord::RemovePunctuation()
